'' =================================================================================================
''
''   File....... jm_button_graph.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2020 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 20 OCT 2020
''
''   {$P1}
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  BR_TERM  = 115_200                                            ' terminal baud rate


con { io pins }

  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }

  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O }


con { app io pins }

  RED_BTN = 26  { I }
  MSB     = 15  { O }
  LSB     =  8  { O }


con

  #true,  ON, OFF
  #false, NO, YES


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal

' * uses cog when loaded


dat { pre-initialized }


var { globals }


pub main | btnstate, t0, t1, level

  setup

  repeat                                                                ' wait for button release
    btnstate := ((btnstate << 1) | ina[RED_BTN]) & %11
    if (btnstate == %00)
      quit

  repeat
    btnstate := ((btnstate << 1) | ina[RED_BTN]) & %11                  ' re-scan

    if (btnstate == %11)                                                ' held down?
      t1 := cnt                                                         ' update button timing
      level := ((t1-t0) / clkfreq) <# 8                                 ' get # of seconds pressed
      outa[MSB..LSB] := %11111111 >> (8-level)                          ' update graph

    else
      t0 := t1 := cnt                                                   ' reset timing
      outa[MSB..LSB] := %00000000                                       ' clear display


  repeat
    waitcnt(0)


pub wait_for_terminal(clear)

  ' Download to RAM with F10
  ' F12 to open PST
  ' Click [Enable]
  ' Press Enter

  term.rxflush
  term.rx
  if (clear)
    term.tx(term#CLS)


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  dira[LSB..MSB] := %11111111                                   ' make LEDs outputs

  term.start(RX1, TX1, %0000, BR_TERM)                          ' start serial for terminal *


con { license }

{{

  MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

}}